<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Form extends ZendX_JQuery_Form
{
	/**
     * Constructs a new Form class and stores the default decorator options
     * @param array $options
     */
    public function __construct($options = null)
    {
        $this->addPrefixPath(
            'DZF_Form_Decorator_',
            'DZF/Form/Decorator/',
            Zend_Form::DECORATOR
        );

        $this->addPrefixPath(
            'DZF_Form_Element_',
            'DZF/Form/Element/',
            Zend_Form::ELEMENT
        );

        $this->addElementPrefixPath(
            'DZF_Validate_',
            'DZF/Validate/',
            'validate'
        );

        $this->setDisableLoadDefaultDecorators(true);
        $this->setDefaultFormDecorators();

        parent::__construct($options);
    }

    /**
     * Add a pre-render functionality to forms so decorators can
     * manipulate forms just before the form gets rendered
     *
     * @param Zend_View_Interface $view
     * @return string
     */
    public function render(Zend_View_Interface $view = null)
    {
        foreach ($this->getDecorators() as $decorator)
        {
            if (method_exists($decorator, 'preRender'))
            {
                $decorator->setElement($this);
                $decorator->preRender();
            }
        }
        return parent::render($view);
    }

    /**
     * Overrides addElement function of Zend_Form to ensure default
     * decorators are set within element
     * @param mixed $element
     * @param string $name
     * @param array $options
     */
    public function addElement($element, $name = null, $options = null)
    {
        parent::addElement($element, $name, $options);

        if ($element instanceof Zend_Form_Element) {
            $this->setDefaultElementDecorators($element);
        } else {
            $this->setDefaultElementDecorators($this->getElement($name));
        }
        return $this;
    }

    public function setDefaultFormDecorators()
    {
        $decorators = $this->getDecorators();
        if (empty($decorators)) {
            $this->setDecorators(
                array(
                    'FormElements',
                    array(
                        'HtmlTag', array(
                            'tag' => 'div',
                            'class' => 'dzf_form'
                        )
                    ),
                    'Form'
                )
            );
        }
    }

    /**
     * Overrides addDisplayGroup function of Zend_Form to ensure
     * that default decorators are set for display groups
     * @param $elements
     * @param $name
     * @param $options
     */
    public function addDisplayGroup(array $elements, $name, $options = null)
    {
        parent::addDisplayGroup($elements, $name, $options);
        $this->setDefaultDisplayGroupDecorators($this->getDisplayGroup($name));
        return $this;
    }

    public function setDefaultDisplayGroupDecorators ($element)
    {
        $element->setDecorators(
            array(
                'FormElements',
                'Fieldset',
                array('HtmlTag', array(
                    'tag' => 'div',
                    'class' => 'dzf_form_displaygroup'
                ))
            )
        );
    }

    /**
     *
     * @param Zend_Form_Element $element
     */
    public function setDefaultElementDecorators($element)
    {
        $element->removeDecorator('DtDdWrapper');
        $element->removeDecorator('HtmlTag');

        // move errors decorator to last position before closing div
        $hasErrorsDecorator = !is_null($element->getDecorator('Errors'));
        if ($hasErrorsDecorator)
        {
            $element->removeDecorator('Errors');
            $element->addDecorator('Errors');
        }

        $element->addDecorators(
            array(
                array(
                    'formElement' => 'HtmlTag',
                    array(
                        'tag' => 'div',
                        'class' => 'dzf_form_element'
                    )
                )
            )
        );

        $labelDecorator = $element->getDecorator('Label');
        if ($labelDecorator)
        {
            $labelDecorator->setTag(null);
            $labelDecorator->setReqSuffix(':');
            $labelDecorator->setOptSuffix(':');
        }
    }

    /**
     * Adds the AjaxValidation FormDecorator
     *
     * @return void
     */
    public function enableAjaxValidation()
    {
        $this->setDefaultFormDecorators();
        $this->addDecorator('AjaxValidation');
    }

    /**
     * @param Zend_Form_Element $element
     * @param string $afterDecoratorName
     * @param Zend_Form_Decorator_Abstract $decorator
     * @param string $decoratorName
     * @return void
     */
    public function addDecoratorAfter(Zend_Form_Element $element, $afterDecoratorName, Zend_Form_Decorator_Abstract $decorator, $decoratorName)
    {
        $oldDecorators = $element->getDecorators();
        $newDecorators = array();
        foreach ($oldDecorators as $key => $value)
        {
            $newDecorators[$key] = $value;
            if ($key == $afterDecoratorName)
            {
                $newDecorators[$decoratorName] = $decorator;
            }
        }

        $element->setDecorators($newDecorators);
    }

    /**
     * @param string $elementName
     * @param array|string $className
     * @return void
     */
    public function addCssClassToContainer($elementName, $className)
    {
        $classesToAdd = array();
        if (is_array($className))
        {
            $classesToAdd = $className;
        }
        else
        {
            $classesToAdd[] = $className;
        }

        $this->getElement($elementName)
            ->getDecorator('HtmlTag')
            ->setOption(
                'class',
                $this->getElement($elementName)
                    ->getDecorator('HtmlTag')
                    ->getOption('class').' '.implode(' ', $classesToAdd)
        );
    }
}